#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-02-08 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = [
	'time_month',
	'time_day',
	'time_hour',
	'time_min',
	'stationid',
	'pos_longitude',
	'pos_latitude',
	'speed',
	'gust',
	'direction',
	'atmpressure',
	'airtemp',
	'dewpoint',
	'visibility',
	'watertemp',
	'reserved',
]

def encode(params, validate=False):
	'''Create a sls_weatherreport binary message payload to pack into an AIS Msg sls_weatherreport.

	Fields in params:
	  - time_month(uint): Time tag of measurement  month 1..12
	  - time_day(uint): Time tag of measurement  day of the month 1..31
	  - time_hour(uint): Time tag of measurement  UTC hours 0..23
	  - time_min(uint): Time tag of measurement  minutes
	  - stationid(aisstr6): Character identifier of the station
	  - pos_longitude(decimal): Location of measurement  East West location
	  - pos_latitude(decimal): Location of measurement  North South location
	  - speed(udecimal): Average wind speed
	  - gust(udecimal): Wind gust
	  - direction(uint): Wind direction
	  - atmpressure(udecimal): Atmospheric pressure
	  - airtemp(decimal): Air temperature
	  - dewpoint(decimal): Dew Point
	  - visibility(udecimal): Visibility
	  - watertemp(decimal): Water Temperature
	  - reserved(uint): Reserved bits for future use (field automatically set to "0")
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['time_min']),6))
	if 'stationid' in params:
		bvList.append(aisstring.encode(params['stationid'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'pos_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['pos_longitude'])*Decimal('60000')),25))
	else:
		bvList.append(binary.bvFromSignedInt(10860000,25))
	if 'pos_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['pos_latitude'])*Decimal('60000')),24))
	else:
		bvList.append(binary.bvFromSignedInt(5460000,24))
	if 'speed' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['speed'])*Decimal('10')))),10))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(1023)),10))
	if 'gust' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['gust'])*Decimal('10')))),10))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(1023)),10))
	if 'direction' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['direction']),9))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=511),9))
	if 'atmpressure' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['atmpressure'])*Decimal('10')))),14))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(163830)),14))
	if 'airtemp' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['airtemp'])*Decimal('10')),10))
	else:
		bvList.append(binary.bvFromSignedInt(-512,10))
	if 'dewpoint' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['dewpoint'])*Decimal('10')),10))
	else:
		bvList.append(binary.bvFromSignedInt(-512,10))
	if 'visibility' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['visibility'])*Decimal('10')))),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(255)),8))
	if 'watertemp' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['watertemp'])*Decimal('10')),10))
	else:
		bvList.append(binary.bvFromSignedInt(-512,10))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),4))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a sls_weatherreport message 

	Fields in params:
	  - time_month(uint): Time tag of measurement  month 1..12
	  - time_day(uint): Time tag of measurement  day of the month 1..31
	  - time_hour(uint): Time tag of measurement  UTC hours 0..23
	  - time_min(uint): Time tag of measurement  minutes
	  - stationid(aisstr6): Character identifier of the station
	  - pos_longitude(decimal): Location of measurement  East West location
	  - pos_latitude(decimal): Location of measurement  North South location
	  - speed(udecimal): Average wind speed
	  - gust(udecimal): Wind gust
	  - direction(uint): Wind direction
	  - atmpressure(udecimal): Atmospheric pressure
	  - airtemp(decimal): Air temperature
	  - dewpoint(decimal): Dew Point
	  - visibility(udecimal): Visibility
	  - watertemp(decimal): Water Temperature
	  - reserved(uint): Reserved bits for future use (field automatically set to "0")
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['time_month']=int(bv[0:4])
	r['time_day']=int(bv[4:9])
	r['time_hour']=int(bv[9:14])
	r['time_min']=int(bv[14:20])
	r['stationid']=aisstring.decode(bv[20:62])
	r['pos_longitude']=Decimal(binary.signedIntFromBV(bv[62:87]))/Decimal('60000')
	r['pos_latitude']=Decimal(binary.signedIntFromBV(bv[87:111]))/Decimal('60000')
	r['speed']=Decimal(int(bv[111:121]))/Decimal('10')
	r['gust']=Decimal(int(bv[121:131]))/Decimal('10')
	r['direction']=int(bv[131:140])
	r['atmpressure']=Decimal(int(bv[140:154]))/Decimal('10')
	r['airtemp']=Decimal(binary.signedIntFromBV(bv[154:164]))/Decimal('10')
	r['dewpoint']=Decimal(binary.signedIntFromBV(bv[164:174]))/Decimal('10')
	r['visibility']=Decimal(int(bv[174:182]))/Decimal('10')
	r['watertemp']=Decimal(binary.signedIntFromBV(bv[182:192]))/Decimal('10')
	r['reserved']=0
	return r

def decodetime_month(bv, validate=False):
	return int(bv[0:4])

def decodetime_day(bv, validate=False):
	return int(bv[4:9])

def decodetime_hour(bv, validate=False):
	return int(bv[9:14])

def decodetime_min(bv, validate=False):
	return int(bv[14:20])

def decodestationid(bv, validate=False):
	return aisstring.decode(bv[20:62])

def decodepos_longitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[62:87]))/Decimal('60000')

def decodepos_latitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[87:111]))/Decimal('60000')

def decodespeed(bv, validate=False):
	return Decimal(int(bv[111:121]))/Decimal('10')

def decodegust(bv, validate=False):
	return Decimal(int(bv[121:131]))/Decimal('10')

def decodedirection(bv, validate=False):
	return int(bv[131:140])

def decodeatmpressure(bv, validate=False):
	return Decimal(int(bv[140:154]))/Decimal('10')

def decodeairtemp(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[154:164]))/Decimal('10')

def decodedewpoint(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[164:174]))/Decimal('10')

def decodevisibility(bv, validate=False):
	return Decimal(int(bv[174:182]))/Decimal('10')

def decodewatertemp(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[182:192]))/Decimal('10')

def decodereserved(bv, validate=False):
	return 0


def printHtml(params, out=sys.stdout):
		out.write("<h3>sls_weatherreport<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>time_month</td>\n")
		out.write("<td>uint</td>\n")
		if 'time_month' in params:
			out.write("	<td>"+str(params['time_month'])+"</td>\n")
			out.write("	<td>"+str(params['time_month'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>time_day</td>\n")
		out.write("<td>uint</td>\n")
		if 'time_day' in params:
			out.write("	<td>"+str(params['time_day'])+"</td>\n")
			out.write("	<td>"+str(params['time_day'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>time_hour</td>\n")
		out.write("<td>uint</td>\n")
		if 'time_hour' in params:
			out.write("	<td>"+str(params['time_hour'])+"</td>\n")
			out.write("	<td>"+str(params['time_hour'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>time_min</td>\n")
		out.write("<td>uint</td>\n")
		if 'time_min' in params:
			out.write("	<td>"+str(params['time_min'])+"</td>\n")
			out.write("	<td>"+str(params['time_min'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>stationid</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'stationid' in params:
			out.write("	<td>"+str(params['stationid'])+"</td>\n")
			out.write("	<td>"+str(params['stationid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>pos_longitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'pos_longitude' in params:
			out.write("	<td>"+str(params['pos_longitude'])+"</td>\n")
			out.write("	<td>"+str(params['pos_longitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>pos_latitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'pos_latitude' in params:
			out.write("	<td>"+str(params['pos_latitude'])+"</td>\n")
			out.write("	<td>"+str(params['pos_latitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>speed</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'speed' in params:
			out.write("	<td>"+str(params['speed'])+"</td>\n")
			if str(params['speed']) in speedDecodeLut:
				out.write("<td>"+speedDecodeLut[str(params['speed'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>kts</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>gust</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'gust' in params:
			out.write("	<td>"+str(params['gust'])+"</td>\n")
			if str(params['gust']) in gustDecodeLut:
				out.write("<td>"+gustDecodeLut[str(params['gust'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>kts</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>direction</td>\n")
		out.write("<td>uint</td>\n")
		if 'direction' in params:
			out.write("	<td>"+str(params['direction'])+"</td>\n")
			out.write("	<td>"+str(params['direction'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>atmpressure</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'atmpressure' in params:
			out.write("	<td>"+str(params['atmpressure'])+"</td>\n")
			out.write("	<td>"+str(params['atmpressure'])+"</td>\n")
		out.write("<td>millibars</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>airtemp</td>\n")
		out.write("<td>decimal</td>\n")
		if 'airtemp' in params:
			out.write("	<td>"+str(params['airtemp'])+"</td>\n")
			if str(params['airtemp']) in airtempDecodeLut:
				out.write("<td>"+airtempDecodeLut[str(params['airtemp'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>Celsius</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dewpoint</td>\n")
		out.write("<td>decimal</td>\n")
		if 'dewpoint' in params:
			out.write("	<td>"+str(params['dewpoint'])+"</td>\n")
			if str(params['dewpoint']) in dewpointDecodeLut:
				out.write("<td>"+dewpointDecodeLut[str(params['dewpoint'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>Celsius</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>visibility</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'visibility' in params:
			out.write("	<td>"+str(params['visibility'])+"</td>\n")
			if str(params['visibility']) in visibilityDecodeLut:
				out.write("<td>"+visibilityDecodeLut[str(params['visibility'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>km</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>watertemp</td>\n")
		out.write("<td>decimal</td>\n")
		if 'watertemp' in params:
			out.write("	<td>"+str(params['watertemp'])+"</td>\n")
			if str(params['watertemp']) in watertempDecodeLut:
				out.write("<td>"+watertempDecodeLut[str(params['watertemp'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>Celsius</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>reserved</td>\n")
		out.write("<td>uint</td>\n")
		if 'reserved' in params:
			out.write("	<td>"+str(params['reserved'])+"</td>\n")
			out.write("	<td>"+str(params['reserved'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")


def printKml(params, out=sys.stdout):
	'''KML (Keyhole Markup Language) for Google Earth, but without the header/footer'''
	out.write("\	<Placemark>\n")
	out.write("\t	<name>"+str(params['stationid'])+"</name>\n")
	out.write("\t\t<description>\n")
	import StringIO
	buf = StringIO.StringIO()
	printHtml(params,buf)
	import cgi
	out.write(cgi.escape(buf.getvalue()))
	out.write("\t\t</description>\n")
	out.write("\t\t<styleUrl>#m_ylw-pushpin_copy0</styleUrl>\n")
	out.write("\t\t<Point>\n")
	out.write("\t\t\t<coordinates>")
	out.write(str(params['pos_longitude']))
	out.write(',')
	out.write(str(params['pos_latitude']))
	out.write(",0</coordinates>\n")
	out.write("\t\t</Point>\n")
	out.write("\t</Placemark>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None):
	'''Print a reserved message to stdout.

	Fields in params:
	  - time_month(uint): Time tag of measurement  month 1..12
	  - time_day(uint): Time tag of measurement  day of the month 1..31
	  - time_hour(uint): Time tag of measurement  UTC hours 0..23
	  - time_min(uint): Time tag of measurement  minutes
	  - stationid(aisstr6): Character identifier of the station
	  - pos_longitude(decimal): Location of measurement  East West location
	  - pos_latitude(decimal): Location of measurement  North South location
	  - speed(udecimal): Average wind speed
	  - gust(udecimal): Wind gust
	  - direction(uint): Wind direction
	  - atmpressure(udecimal): Atmospheric pressure
	  - airtemp(decimal): Air temperature
	  - dewpoint(decimal): Dew Point
	  - visibility(udecimal): Visibility
	  - watertemp(decimal): Water Temperature
	  - reserved(uint): Reserved bits for future use (field automatically set to "0")
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("reserved:\n")
		if 'time_month' in params: out.write("	time_month:     "+str(params['time_month'])+"\n")
		if 'time_day' in params: out.write("	time_day:       "+str(params['time_day'])+"\n")
		if 'time_hour' in params: out.write("	time_hour:      "+str(params['time_hour'])+"\n")
		if 'time_min' in params: out.write("	time_min:       "+str(params['time_min'])+"\n")
		if 'stationid' in params: out.write("	stationid:      "+str(params['stationid'])+"\n")
		if 'pos_longitude' in params: out.write("	pos_longitude:  "+str(params['pos_longitude'])+"\n")
		if 'pos_latitude' in params: out.write("	pos_latitude:   "+str(params['pos_latitude'])+"\n")
		if 'speed' in params: out.write("	speed:          "+str(params['speed'])+"\n")
		if 'gust' in params: out.write("	gust:           "+str(params['gust'])+"\n")
		if 'direction' in params: out.write("	direction:      "+str(params['direction'])+"\n")
		if 'atmpressure' in params: out.write("	atmpressure:    "+str(params['atmpressure'])+"\n")
		if 'airtemp' in params: out.write("	airtemp:        "+str(params['airtemp'])+"\n")
		if 'dewpoint' in params: out.write("	dewpoint:       "+str(params['dewpoint'])+"\n")
		if 'visibility' in params: out.write("	visibility:     "+str(params['visibility'])+"\n")
		if 'watertemp' in params: out.write("	watertemp:      "+str(params['watertemp'])+"\n")
		if 'reserved' in params: out.write("	reserved:       "+str(params['reserved'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	elif 'sql'==format:
		sqlInsertStr(params,out)
	elif 'kml'==format:
		printKml(params,out)
	elif 'kml-full'==format:
		out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		out.write("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n")
		out.write("<Document>\n")
		out.write("	<name>sls_weatherreport</name>\n")
		printKml(params,out)
		out.write("</Document>\n")
		out.write("</kml>\n")
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

speedEncodeLut = {
	'102.2 kts or greater':'102.2',
	} #speedEncodeLut

speedDecodeLut = {
	'102.2':'102.2 kts or greater',
	} # speedEncodeLut

gustEncodeLut = {
	'102.2 kts or greater':'102.2',
	} #gustEncodeLut

gustDecodeLut = {
	'102.2':'102.2 kts or greater',
	} # gustEncodeLut

airtempEncodeLut = {
	'-51.1 degrees C or lower':'-51.1',
	'51.1 degrees C or greater':'51.1',
	} #airtempEncodeLut

airtempDecodeLut = {
	'-51.1':'-51.1 degrees C or lower',
	'51.1':'51.1 degrees C or greater',
	} # airtempEncodeLut

dewpointEncodeLut = {
	'-51.1 degrees C or lower':'-51.1',
	'51.1 degrees C or greater':'51.1',
	} #dewpointEncodeLut

dewpointDecodeLut = {
	'-51.1':'-51.1 degrees C or lower',
	'51.1':'51.1 degrees C or greater',
	} # dewpointEncodeLut

visibilityEncodeLut = {
	'25.4 km or greater':'25.4',
	} #visibilityEncodeLut

visibilityDecodeLut = {
	'25.4':'25.4 km or greater',
	} # visibilityEncodeLut

watertempEncodeLut = {
	'-51.1 degrees C or lower':'-51.1',
	'51.1 degrees C or greater':'51.1',
	} #watertempEncodeLut

watertempDecodeLut = {
	'-51.1':'-51.1 degrees C or lower',
	'51.1':'51.1 degrees C or greater',
	} # watertempEncodeLut

######################################################################
# SQL SUPPORT
######################################################################

def sqlCreateStr(outfile=sys.stdout, fields=None, extraFields=None, addCoastGuardFields=True):
	'''
	Return the SQL CREATE command for this message type
	@param outfile: file like object to print to.
	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlCreate(fields,extraFields,addCoastGuardFields)))

def sqlCreate(fields=None, extraFields=None, addCoastGuardFields=True):
	'''
	Return the sqlhelp object to create the table.

	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@return: An object that can be used to generate a return
	@rtype: sqlhelp.create
	'''
	if None == fields: fields = fieldList
	import sqlhelp
	c = sqlhelp.create('sls_weatherreport')
	if 'time_month' in fields: c.addInt ('time_month')
	if 'time_day' in fields: c.addInt ('time_day')
	if 'time_hour' in fields: c.addInt ('time_hour')
	if 'time_min' in fields: c.addInt ('time_min')
	if 'stationid' in fields: c.addVarChar('stationid',7)
	if 'pos_longitude' in fields: c.addDecimal('pos_longitude',7,4)
	if 'pos_latitude' in fields: c.addDecimal('pos_latitude',7,4)
	if 'speed' in fields: c.addDecimal('speed',4,1)
	if 'gust' in fields: c.addDecimal('gust',4,1)
	if 'direction' in fields: c.addInt ('direction')
	if 'atmpressure' in fields: c.addDecimal('atmpressure',5,1)
	if 'airtemp' in fields: c.addDecimal('airtemp',4,1)
	if 'dewpoint' in fields: c.addDecimal('dewpoint',4,1)
	if 'visibility' in fields: c.addDecimal('visibility',3,1)
	if 'watertemp' in fields: c.addDecimal('watertemp',4,1)
	if 'reserved' in fields: c.addInt ('reserved')

	if addCoastGuardFields:
		# c.addInt('cg_rssi')     # Relative signal strength indicator
		# c.addInt('cg_d')        # dBm receive strength
		# c.addInt('cg_T')        # Receive timestamp from the AIS equipment
		# c.addInt('cg_S')        # Slot received in
		# c.addVarChar('cg_x',10) # Idonno
		c.addVarChar('cg_r',15)   # Receiver station ID  -  should usually be an MMSI, but sometimes is a string
		c.addInt('cg_timestamp')  # UTC seconds since the epoch
		# FIX: maybe an actually time field?

	return c

def sqlInsertStr(params, outfile=sys.stdout, extraParams=None):
	'''
	Return the SQL CREATE command for this message type
	@param params: dictionary of values keyed by field name
	@param outfile: file like object to print to.
	@param extraParams: A sequence of tuples containing (name,sql type) for additional fields
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlInsert(params,extraParams)))


def sqlInsert(params,extraParams=None):
	'''
	Give the SQL insert statement
	@param params: dict keyed by field name of values
	@param extraParams: any extra fields that you have created beyond the normal ais message fields
	@rtype: sqlhelp.insert
	@return: insert class instance
	@todo: allow optional type checking of params?
	@warning: this will take invalid keys happily and do what???
	'''
	import sqlhelp
	i = sqlhelp.insert('sls_weatherreport')
	for key in params: 
		#i.add(key,params[key])
		if type(params[key])==Decimal: i.add(key,float(params[key]))
		else: i.add(key,params[key])
	if None != extraParams:
		for key in extraParams: 
			i.add(key,extraParams[key])

	return i


######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['time_month'] = 2
	params['time_day'] = 28
	params['time_hour'] = 23
	params['time_min'] = 45
	params['stationid'] = 'A345678'
	params['pos_longitude'] = Decimal('-122.16328')
	params['pos_latitude'] = Decimal('37.42446')
	params['speed'] = Decimal('0.7')
	params['gust'] = Decimal('0.7')
	params['direction'] = 41
	params['atmpressure'] = Decimal('0.7')
	params['airtemp'] = Decimal('-8')
	params['dewpoint'] = Decimal('-7')
	params['visibility'] = Decimal('12.3')
	params['watertemp'] = Decimal('-0.2')
	params['reserved'] = 0

	return params

class Testsls_weatherreport(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the sls_weatherreport message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['time_month'],params['time_month'])
		self.failUnlessEqual(r['time_day'],params['time_day'])
		self.failUnlessEqual(r['time_hour'],params['time_hour'])
		self.failUnlessEqual(r['time_min'],params['time_min'])
		self.failUnlessEqual(r['stationid'],params['stationid'])
		self.failUnlessAlmostEqual(r['pos_longitude'],params['pos_longitude'],4)
		self.failUnlessAlmostEqual(r['pos_latitude'],params['pos_latitude'],4)
		self.failUnlessAlmostEqual(r['speed'],params['speed'],1)
		self.failUnlessAlmostEqual(r['gust'],params['gust'],1)
		self.failUnlessEqual(r['direction'],params['direction'])
		self.failUnlessAlmostEqual(r['atmpressure'],params['atmpressure'],1)
		self.failUnlessAlmostEqual(r['airtemp'],params['airtemp'],1)
		self.failUnlessAlmostEqual(r['dewpoint'],params['dewpoint'],1)
		self.failUnlessAlmostEqual(r['visibility'],params['visibility'],1)
		self.failUnlessAlmostEqual(r['watertemp'],params['watertemp'],1)
		self.failUnlessEqual(r['reserved'],params['reserved'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "sls_weatherreport" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "sls_weatherreport" AIS message')
	parser.add_option('--time_month-field', dest='time_monthField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--time_day-field', dest='time_dayField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--time_hour-field', dest='time_hourField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--time_min-field', dest='time_minField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--stationid-field', dest='stationidField',default='@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--pos_longitude-field', dest='pos_longitudeField',default=Decimal('181'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--pos_latitude-field', dest='pos_latitudeField',default=Decimal('91'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--speed-field', dest='speedField',default=Decimal('102.3'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--gust-field', dest='gustField',default=Decimal('102.3'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--direction-field', dest='directionField',default=511,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--atmpressure-field', dest='atmpressureField',default=Decimal('16383'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--airtemp-field', dest='airtempField',default=Decimal('-51.2'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dewpoint-field', dest='dewpointField',default=Decimal('-51.2'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--visibility-field', dest='visibilityField',default=Decimal('25.5'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--watertemp-field', dest='watertempField',default=Decimal('-51.2'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv','sql' , 'kml','kml-full')
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	parser.add_option('-c','--sql-create',dest='sqlCreate',default=False,action='store_true',
			  help='Print out an sql create command for the table.')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.time_monthField: parser.error("missing value for time_monthField")
		if None==options.time_dayField: parser.error("missing value for time_dayField")
		if None==options.time_hourField: parser.error("missing value for time_hourField")
		if None==options.time_minField: parser.error("missing value for time_minField")
		if None==options.stationidField: parser.error("missing value for stationidField")
		if None==options.pos_longitudeField: parser.error("missing value for pos_longitudeField")
		if None==options.pos_latitudeField: parser.error("missing value for pos_latitudeField")
		if None==options.speedField: parser.error("missing value for speedField")
		if None==options.gustField: parser.error("missing value for gustField")
		if None==options.directionField: parser.error("missing value for directionField")
		if None==options.atmpressureField: parser.error("missing value for atmpressureField")
		if None==options.airtempField: parser.error("missing value for airtempField")
		if None==options.dewpointField: parser.error("missing value for dewpointField")
		if None==options.visibilityField: parser.error("missing value for visibilityField")
		if None==options.watertempField: parser.error("missing value for watertempField")
		msgDict={
			'time_month': options.time_monthField,
			'time_day': options.time_dayField,
			'time_hour': options.time_hourField,
			'time_min': options.time_minField,
			'stationid': options.stationidField,
			'pos_longitude': options.pos_longitudeField,
			'pos_latitude': options.pos_latitudeField,
			'speed': options.speedField,
			'gust': options.gustField,
			'direction': options.directionField,
			'atmpressure': options.atmpressureField,
			'airtemp': options.airtempField,
			'dewpoint': options.dewpointField,
			'visibility': options.visibilityField,
			'watertemp': options.watertempField,
			'reserved': '0',
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')


	if options.sqlCreate:
		sqlCreateStr(outfile,options.fieldList)

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType,fieldList=options.fieldList)
