#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-02-08 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = [
	'dac',
	'fid',
	'efid',
	'timetag_month',
	'timetag_day',
	'timetag_hour',
	'timetag_min',
	'timetag_sec',
	'stationid',
	'stationloc_longitude',
	'stationloc_latitude',
	'waterlevel',
	'datum',
	'sigma',
	'o',
	'levelinferred',
	'flat_tolerance_exceeded',
	'rate_tolerance_exceeded',
	'temp_tolerance_exceeded',
	'expected_height_exceeded',
	'link_down',
	'timeLastMeasured_month',
	'timeLastMeasured_day',
	'timeLastMeasured_hour',
	'timeLastMeasured_min',
	'timeLastMeasured_sec',
]

def encode(params, validate=False):
	'''Create a waterlevel binary message payload to pack into an AIS Msg waterlevel.

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (field automatically set to "1")
	  - timetag_month(uint): Time the measurement represents  month 1..12
	  - timetag_day(uint): Time the measurement represents  day of the month 1..31
	  - timetag_hour(uint): Time the measurement represents  UTC hours 0..23
	  - timetag_min(uint): Time the measurement represents  minutes
	  - timetag_sec(uint): Time the measurement represents  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - stationloc_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - stationloc_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - sigma(float): Standard deviation of 1 second samples used to compute the water level height
	  - o(uint): Count of number of samples that fall outside a 3-sigma band about the mean
	  - levelinferred(bool): indicates that the water level value has been inferred
	  - flat_tolerance_exceeded(bool): flat tolerance limit was exceeded.  Need better descr
	  - rate_tolerance_exceeded(bool): rate of change tolerance limit was exceeded
	  - temp_tolerance_exceeded(bool): temperature difference tolerance limit was exceeded
	  - expected_height_exceeded(bool): either the maximum or minimum expected water level height limit was exceeded
	  - link_down(bool): Unable to communicate with the tide system.  All data invalid
	  - timeLastMeasured_month(uint): Time since last measured value was available  month 1..12
	  - timeLastMeasured_day(uint): Time since last measured value was available  day of the month 1..31
	  - timeLastMeasured_hour(uint): Time since last measured value was available  UTC hours 0..23
	  - timeLastMeasured_min(uint): Time since last measured value was available  minutes
	  - timeLastMeasured_sec(uint): Time since last measured value was available  seconds
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=366),16))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),12))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_min']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_sec']),6))
	if 'stationid' in params:
		bvList.append(aisstring.encode(params['stationid'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'stationloc_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['stationloc_longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'stationloc_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['stationloc_latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'waterlevel' in params:
		bvList.append(binary.bvFromSignedInt(params['waterlevel'],16))
	else:
		bvList.append(binary.bvFromSignedInt(-32768,16))
	if 'datum' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['datum']),5))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=31),5))
	bvList.append(binary.float2bitvec(params['sigma']))
	if 'o' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['o']),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=255),8))
	if params["levelinferred"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["flat_tolerance_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["rate_tolerance_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["temp_tolerance_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["expected_height_exceeded"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	if params["link_down"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_min']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timeLastMeasured_sec']),6))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a waterlevel message 

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (field automatically set to "1")
	  - timetag_month(uint): Time the measurement represents  month 1..12
	  - timetag_day(uint): Time the measurement represents  day of the month 1..31
	  - timetag_hour(uint): Time the measurement represents  UTC hours 0..23
	  - timetag_min(uint): Time the measurement represents  minutes
	  - timetag_sec(uint): Time the measurement represents  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - stationloc_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - stationloc_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - sigma(float): Standard deviation of 1 second samples used to compute the water level height
	  - o(uint): Count of number of samples that fall outside a 3-sigma band about the mean
	  - levelinferred(bool): indicates that the water level value has been inferred
	  - flat_tolerance_exceeded(bool): flat tolerance limit was exceeded.  Need better descr
	  - rate_tolerance_exceeded(bool): rate of change tolerance limit was exceeded
	  - temp_tolerance_exceeded(bool): temperature difference tolerance limit was exceeded
	  - expected_height_exceeded(bool): either the maximum or minimum expected water level height limit was exceeded
	  - link_down(bool): Unable to communicate with the tide system.  All data invalid
	  - timeLastMeasured_month(uint): Time since last measured value was available  month 1..12
	  - timeLastMeasured_day(uint): Time since last measured value was available  day of the month 1..31
	  - timeLastMeasured_hour(uint): Time since last measured value was available  UTC hours 0..23
	  - timeLastMeasured_min(uint): Time since last measured value was available  minutes
	  - timeLastMeasured_sec(uint): Time since last measured value was available  seconds
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['dac']=366
	r['fid']=1
	r['efid']=1
	r['timetag_month']=int(bv[32:36])
	r['timetag_day']=int(bv[36:41])
	r['timetag_hour']=int(bv[41:46])
	r['timetag_min']=int(bv[46:52])
	r['timetag_sec']=int(bv[52:58])
	r['stationid']=aisstring.decode(bv[58:100])
	r['stationloc_longitude']=Decimal(binary.signedIntFromBV(bv[100:128]))/Decimal('600000')
	r['stationloc_latitude']=Decimal(binary.signedIntFromBV(bv[128:155]))/Decimal('600000')
	r['waterlevel']=binary.signedIntFromBV(bv[155:171])
	r['datum']=int(bv[171:176])
	r['sigma']=binary.bitvec2float(bv[176:208])
	r['o']=int(bv[208:216])
	r['levelinferred']=bool(int(bv[216:217]))
	r['flat_tolerance_exceeded']=bool(int(bv[217:218]))
	r['rate_tolerance_exceeded']=bool(int(bv[218:219]))
	r['temp_tolerance_exceeded']=bool(int(bv[219:220]))
	r['expected_height_exceeded']=bool(int(bv[220:221]))
	r['link_down']=bool(int(bv[221:222]))
	r['timeLastMeasured_month']=int(bv[222:226])
	r['timeLastMeasured_day']=int(bv[226:231])
	r['timeLastMeasured_hour']=int(bv[231:236])
	r['timeLastMeasured_min']=int(bv[236:242])
	r['timeLastMeasured_sec']=int(bv[242:248])
	return r

def decodedac(bv, validate=False):
	return 366

def decodefid(bv, validate=False):
	return 1

def decodeefid(bv, validate=False):
	return 1

def decodetimetag_month(bv, validate=False):
	return int(bv[32:36])

def decodetimetag_day(bv, validate=False):
	return int(bv[36:41])

def decodetimetag_hour(bv, validate=False):
	return int(bv[41:46])

def decodetimetag_min(bv, validate=False):
	return int(bv[46:52])

def decodetimetag_sec(bv, validate=False):
	return int(bv[52:58])

def decodestationid(bv, validate=False):
	return aisstring.decode(bv[58:100])

def decodestationloc_longitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[100:128]))/Decimal('600000')

def decodestationloc_latitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[128:155]))/Decimal('600000')

def decodewaterlevel(bv, validate=False):
	return binary.signedIntFromBV(bv[155:171])

def decodedatum(bv, validate=False):
	return int(bv[171:176])

def decodesigma(bv, validate=False):
	return binary.bitvec2float(bv[176:208])

def decodeo(bv, validate=False):
	return int(bv[208:216])

def decodelevelinferred(bv, validate=False):
	return bool(int(bv[216:217]))

def decodeflat_tolerance_exceeded(bv, validate=False):
	return bool(int(bv[217:218]))

def decoderate_tolerance_exceeded(bv, validate=False):
	return bool(int(bv[218:219]))

def decodetemp_tolerance_exceeded(bv, validate=False):
	return bool(int(bv[219:220]))

def decodeexpected_height_exceeded(bv, validate=False):
	return bool(int(bv[220:221]))

def decodelink_down(bv, validate=False):
	return bool(int(bv[221:222]))

def decodetimeLastMeasured_month(bv, validate=False):
	return int(bv[222:226])

def decodetimeLastMeasured_day(bv, validate=False):
	return int(bv[226:231])

def decodetimeLastMeasured_hour(bv, validate=False):
	return int(bv[231:236])

def decodetimeLastMeasured_min(bv, validate=False):
	return int(bv[236:242])

def decodetimeLastMeasured_sec(bv, validate=False):
	return int(bv[242:248])


def printHtml(params, out=sys.stdout):
		out.write("<h3>waterlevel<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dac</td>\n")
		out.write("<td>uint</td>\n")
		if 'dac' in params:
			out.write("	<td>"+str(params['dac'])+"</td>\n")
			out.write("	<td>"+str(params['dac'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>fid</td>\n")
		out.write("<td>uint</td>\n")
		if 'fid' in params:
			out.write("	<td>"+str(params['fid'])+"</td>\n")
			out.write("	<td>"+str(params['fid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>efid</td>\n")
		out.write("<td>uint</td>\n")
		if 'efid' in params:
			out.write("	<td>"+str(params['efid'])+"</td>\n")
			out.write("	<td>"+str(params['efid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_month</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_month' in params:
			out.write("	<td>"+str(params['timetag_month'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_month'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_day</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_day' in params:
			out.write("	<td>"+str(params['timetag_day'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_day'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_hour</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_hour' in params:
			out.write("	<td>"+str(params['timetag_hour'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_hour'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_min</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_min' in params:
			out.write("	<td>"+str(params['timetag_min'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_min'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_sec</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_sec' in params:
			out.write("	<td>"+str(params['timetag_sec'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_sec'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>stationid</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'stationid' in params:
			out.write("	<td>"+str(params['stationid'])+"</td>\n")
			out.write("	<td>"+str(params['stationid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>stationloc_longitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'stationloc_longitude' in params:
			out.write("	<td>"+str(params['stationloc_longitude'])+"</td>\n")
			out.write("	<td>"+str(params['stationloc_longitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>stationloc_latitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'stationloc_latitude' in params:
			out.write("	<td>"+str(params['stationloc_latitude'])+"</td>\n")
			out.write("	<td>"+str(params['stationloc_latitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>waterlevel</td>\n")
		out.write("<td>int</td>\n")
		if 'waterlevel' in params:
			out.write("	<td>"+str(params['waterlevel'])+"</td>\n")
			out.write("	<td>"+str(params['waterlevel'])+"</td>\n")
		out.write("<td>cm</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>datum</td>\n")
		out.write("<td>uint</td>\n")
		if 'datum' in params:
			out.write("	<td>"+str(params['datum'])+"</td>\n")
			if str(params['datum']) in datumDecodeLut:
				out.write("<td>"+datumDecodeLut[str(params['datum'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>sigma</td>\n")
		out.write("<td>float</td>\n")
		if 'sigma' in params:
			out.write("	<td>"+str(params['sigma'])+"</td>\n")
			out.write("	<td>"+str(params['sigma'])+"</td>\n")
		out.write("<td>m</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>o</td>\n")
		out.write("<td>uint</td>\n")
		if 'o' in params:
			out.write("	<td>"+str(params['o'])+"</td>\n")
			out.write("	<td>"+str(params['o'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>levelinferred</td>\n")
		out.write("<td>bool</td>\n")
		if 'levelinferred' in params:
			out.write("	<td>"+str(params['levelinferred'])+"</td>\n")
			out.write("	<td>"+str(params['levelinferred'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>flat_tolerance_exceeded</td>\n")
		out.write("<td>bool</td>\n")
		if 'flat_tolerance_exceeded' in params:
			out.write("	<td>"+str(params['flat_tolerance_exceeded'])+"</td>\n")
			out.write("	<td>"+str(params['flat_tolerance_exceeded'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>rate_tolerance_exceeded</td>\n")
		out.write("<td>bool</td>\n")
		if 'rate_tolerance_exceeded' in params:
			out.write("	<td>"+str(params['rate_tolerance_exceeded'])+"</td>\n")
			out.write("	<td>"+str(params['rate_tolerance_exceeded'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>temp_tolerance_exceeded</td>\n")
		out.write("<td>bool</td>\n")
		if 'temp_tolerance_exceeded' in params:
			out.write("	<td>"+str(params['temp_tolerance_exceeded'])+"</td>\n")
			out.write("	<td>"+str(params['temp_tolerance_exceeded'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>expected_height_exceeded</td>\n")
		out.write("<td>bool</td>\n")
		if 'expected_height_exceeded' in params:
			out.write("	<td>"+str(params['expected_height_exceeded'])+"</td>\n")
			out.write("	<td>"+str(params['expected_height_exceeded'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>link_down</td>\n")
		out.write("<td>bool</td>\n")
		if 'link_down' in params:
			out.write("	<td>"+str(params['link_down'])+"</td>\n")
			out.write("	<td>"+str(params['link_down'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timeLastMeasured_month</td>\n")
		out.write("<td>uint</td>\n")
		if 'timeLastMeasured_month' in params:
			out.write("	<td>"+str(params['timeLastMeasured_month'])+"</td>\n")
			out.write("	<td>"+str(params['timeLastMeasured_month'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timeLastMeasured_day</td>\n")
		out.write("<td>uint</td>\n")
		if 'timeLastMeasured_day' in params:
			out.write("	<td>"+str(params['timeLastMeasured_day'])+"</td>\n")
			out.write("	<td>"+str(params['timeLastMeasured_day'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timeLastMeasured_hour</td>\n")
		out.write("<td>uint</td>\n")
		if 'timeLastMeasured_hour' in params:
			out.write("	<td>"+str(params['timeLastMeasured_hour'])+"</td>\n")
			out.write("	<td>"+str(params['timeLastMeasured_hour'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timeLastMeasured_min</td>\n")
		out.write("<td>uint</td>\n")
		if 'timeLastMeasured_min' in params:
			out.write("	<td>"+str(params['timeLastMeasured_min'])+"</td>\n")
			out.write("	<td>"+str(params['timeLastMeasured_min'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timeLastMeasured_sec</td>\n")
		out.write("<td>uint</td>\n")
		if 'timeLastMeasured_sec' in params:
			out.write("	<td>"+str(params['timeLastMeasured_sec'])+"</td>\n")
			out.write("	<td>"+str(params['timeLastMeasured_sec'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")


def printKml(params, out=sys.stdout):
	'''KML (Keyhole Markup Language) for Google Earth, but without the header/footer'''
	out.write("\	<Placemark>\n")
	out.write("\t	<name>"+str(params['stationid'])+"</name>\n")
	out.write("\t\t<description>\n")
	import StringIO
	buf = StringIO.StringIO()
	printHtml(params,buf)
	import cgi
	out.write(cgi.escape(buf.getvalue()))
	out.write("\t\t</description>\n")
	out.write("\t\t<styleUrl>#m_ylw-pushpin_copy0</styleUrl>\n")
	out.write("\t\t<Point>\n")
	out.write("\t\t\t<coordinates>")
	out.write(str(params['stationloc_longitude']))
	out.write(',')
	out.write(str(params['stationloc_latitude']))
	out.write(",0</coordinates>\n")
	out.write("\t\t</Point>\n")
	out.write("\t</Placemark>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None):
	'''Print a timeLastMeasured_sec message to stdout.

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (field automatically set to "1")
	  - timetag_month(uint): Time the measurement represents  month 1..12
	  - timetag_day(uint): Time the measurement represents  day of the month 1..31
	  - timetag_hour(uint): Time the measurement represents  UTC hours 0..23
	  - timetag_min(uint): Time the measurement represents  minutes
	  - timetag_sec(uint): Time the measurement represents  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - stationloc_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - stationloc_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - waterlevel(int): Water level in centimeters
	  - datum(uint): What reference datum applies to the value
	  - sigma(float): Standard deviation of 1 second samples used to compute the water level height
	  - o(uint): Count of number of samples that fall outside a 3-sigma band about the mean
	  - levelinferred(bool): indicates that the water level value has been inferred
	  - flat_tolerance_exceeded(bool): flat tolerance limit was exceeded.  Need better descr
	  - rate_tolerance_exceeded(bool): rate of change tolerance limit was exceeded
	  - temp_tolerance_exceeded(bool): temperature difference tolerance limit was exceeded
	  - expected_height_exceeded(bool): either the maximum or minimum expected water level height limit was exceeded
	  - link_down(bool): Unable to communicate with the tide system.  All data invalid
	  - timeLastMeasured_month(uint): Time since last measured value was available  month 1..12
	  - timeLastMeasured_day(uint): Time since last measured value was available  day of the month 1..31
	  - timeLastMeasured_hour(uint): Time since last measured value was available  UTC hours 0..23
	  - timeLastMeasured_min(uint): Time since last measured value was available  minutes
	  - timeLastMeasured_sec(uint): Time since last measured value was available  seconds
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("timeLastMeasured_sec:\n")
		if 'dac' in params: out.write("	dac:                       "+str(params['dac'])+"\n")
		if 'fid' in params: out.write("	fid:                       "+str(params['fid'])+"\n")
		if 'efid' in params: out.write("	efid:                      "+str(params['efid'])+"\n")
		if 'timetag_month' in params: out.write("	timetag_month:             "+str(params['timetag_month'])+"\n")
		if 'timetag_day' in params: out.write("	timetag_day:               "+str(params['timetag_day'])+"\n")
		if 'timetag_hour' in params: out.write("	timetag_hour:              "+str(params['timetag_hour'])+"\n")
		if 'timetag_min' in params: out.write("	timetag_min:               "+str(params['timetag_min'])+"\n")
		if 'timetag_sec' in params: out.write("	timetag_sec:               "+str(params['timetag_sec'])+"\n")
		if 'stationid' in params: out.write("	stationid:                 "+str(params['stationid'])+"\n")
		if 'stationloc_longitude' in params: out.write("	stationloc_longitude:      "+str(params['stationloc_longitude'])+"\n")
		if 'stationloc_latitude' in params: out.write("	stationloc_latitude:       "+str(params['stationloc_latitude'])+"\n")
		if 'waterlevel' in params: out.write("	waterlevel:                "+str(params['waterlevel'])+"\n")
		if 'datum' in params: out.write("	datum:                     "+str(params['datum'])+"\n")
		if 'sigma' in params: out.write("	sigma:                     "+str(params['sigma'])+"\n")
		if 'o' in params: out.write("	o:                         "+str(params['o'])+"\n")
		if 'levelinferred' in params: out.write("	levelinferred:             "+str(params['levelinferred'])+"\n")
		if 'flat_tolerance_exceeded' in params: out.write("	flat_tolerance_exceeded:   "+str(params['flat_tolerance_exceeded'])+"\n")
		if 'rate_tolerance_exceeded' in params: out.write("	rate_tolerance_exceeded:   "+str(params['rate_tolerance_exceeded'])+"\n")
		if 'temp_tolerance_exceeded' in params: out.write("	temp_tolerance_exceeded:   "+str(params['temp_tolerance_exceeded'])+"\n")
		if 'expected_height_exceeded' in params: out.write("	expected_height_exceeded:  "+str(params['expected_height_exceeded'])+"\n")
		if 'link_down' in params: out.write("	link_down:                 "+str(params['link_down'])+"\n")
		if 'timeLastMeasured_month' in params: out.write("	timeLastMeasured_month:    "+str(params['timeLastMeasured_month'])+"\n")
		if 'timeLastMeasured_day' in params: out.write("	timeLastMeasured_day:      "+str(params['timeLastMeasured_day'])+"\n")
		if 'timeLastMeasured_hour' in params: out.write("	timeLastMeasured_hour:     "+str(params['timeLastMeasured_hour'])+"\n")
		if 'timeLastMeasured_min' in params: out.write("	timeLastMeasured_min:      "+str(params['timeLastMeasured_min'])+"\n")
		if 'timeLastMeasured_sec' in params: out.write("	timeLastMeasured_sec:      "+str(params['timeLastMeasured_sec'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	elif 'sql'==format:
		sqlInsertStr(params,out)
	elif 'kml'==format:
		printKml(params,out)
	elif 'kml-full'==format:
		out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		out.write("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n")
		out.write("<Document>\n")
		out.write("	<name>waterlevel</name>\n")
		printKml(params,out)
		out.write("</Document>\n")
		out.write("</kml>\n")
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

datumEncodeLut = {
	'MLLW':'0',
	'IGLD-85':'1',
	'WaterDepth':'2',
	'STND':'3',
	'MHW':'4',
	'MLS':'5',
	'NGVD':'6',
	'NAVD':'7',
	'WGS-84':'8',
	'LAT':'9',
	} #datumEncodeLut

datumDecodeLut = {
	'0':'MLLW',
	'1':'IGLD-85',
	'2':'WaterDepth',
	'3':'STND',
	'4':'MHW',
	'5':'MLS',
	'6':'NGVD',
	'7':'NAVD',
	'8':'WGS-84',
	'9':'LAT',
	} # datumEncodeLut

######################################################################
# SQL SUPPORT
######################################################################

def sqlCreateStr(outfile=sys.stdout, fields=None, extraFields=None, addCoastGuardFields=True):
	'''
	Return the SQL CREATE command for this message type
	@param outfile: file like object to print to.
	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlCreate(fields,extraFields,addCoastGuardFields)))

def sqlCreate(fields=None, extraFields=None, addCoastGuardFields=True):
	'''
	Return the sqlhelp object to create the table.

	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@return: An object that can be used to generate a return
	@rtype: sqlhelp.create
	'''
	if None == fields: fields = fieldList
	import sqlhelp
	c = sqlhelp.create('waterlevel')
	if 'dac' in fields: c.addInt ('dac')
	if 'fid' in fields: c.addInt ('fid')
	if 'efid' in fields: c.addInt ('efid')
	if 'timetag_month' in fields: c.addInt ('timetag_month')
	if 'timetag_day' in fields: c.addInt ('timetag_day')
	if 'timetag_hour' in fields: c.addInt ('timetag_hour')
	if 'timetag_min' in fields: c.addInt ('timetag_min')
	if 'timetag_sec' in fields: c.addInt ('timetag_sec')
	if 'stationid' in fields: c.addVarChar('stationid',7)
	if 'stationloc_longitude' in fields: c.addDecimal('stationloc_longitude',8,5)
	if 'stationloc_latitude' in fields: c.addDecimal('stationloc_latitude',8,5)
	if 'waterlevel' in fields: c.addInt ('waterlevel')
	if 'datum' in fields: c.addInt ('datum')
	if 'sigma' in fields: c.addReal('sigma')
	if 'o' in fields: c.addInt ('o')
	if 'levelinferred' in fields: c.addBool('levelinferred')
	if 'flat_tolerance_exceeded' in fields: c.addBool('flat_tolerance_exceeded')
	if 'rate_tolerance_exceeded' in fields: c.addBool('rate_tolerance_exceeded')
	if 'temp_tolerance_exceeded' in fields: c.addBool('temp_tolerance_exceeded')
	if 'expected_height_exceeded' in fields: c.addBool('expected_height_exceeded')
	if 'link_down' in fields: c.addBool('link_down')
	if 'timeLastMeasured_month' in fields: c.addInt ('timeLastMeasured_month')
	if 'timeLastMeasured_day' in fields: c.addInt ('timeLastMeasured_day')
	if 'timeLastMeasured_hour' in fields: c.addInt ('timeLastMeasured_hour')
	if 'timeLastMeasured_min' in fields: c.addInt ('timeLastMeasured_min')
	if 'timeLastMeasured_sec' in fields: c.addInt ('timeLastMeasured_sec')

	if addCoastGuardFields:
		# c.addInt('cg_rssi')     # Relative signal strength indicator
		# c.addInt('cg_d')        # dBm receive strength
		# c.addInt('cg_T')        # Receive timestamp from the AIS equipment
		# c.addInt('cg_S')        # Slot received in
		# c.addVarChar('cg_x',10) # Idonno
		c.addVarChar('cg_r',15)   # Receiver station ID  -  should usually be an MMSI, but sometimes is a string
		c.addInt('cg_timestamp')  # UTC seconds since the epoch
		# FIX: maybe an actually time field?

	return c

def sqlInsertStr(params, outfile=sys.stdout, extraParams=None):
	'''
	Return the SQL CREATE command for this message type
	@param params: dictionary of values keyed by field name
	@param outfile: file like object to print to.
	@param extraParams: A sequence of tuples containing (name,sql type) for additional fields
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlInsert(params,extraParams)))


def sqlInsert(params,extraParams=None):
	'''
	Give the SQL insert statement
	@param params: dict keyed by field name of values
	@param extraParams: any extra fields that you have created beyond the normal ais message fields
	@rtype: sqlhelp.insert
	@return: insert class instance
	@todo: allow optional type checking of params?
	@warning: this will take invalid keys happily and do what???
	'''
	import sqlhelp
	i = sqlhelp.insert('waterlevel')
	for key in params: 
		#i.add(key,params[key])
		if type(params[key])==Decimal: i.add(key,float(params[key]))
		else: i.add(key,params[key])
	if None != extraParams:
		for key in extraParams: 
			i.add(key,extraParams[key])

	return i


######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['dac'] = 366
	params['fid'] = 1
	params['efid'] = 1
	params['timetag_month'] = 2
	params['timetag_day'] = 28
	params['timetag_hour'] = 23
	params['timetag_min'] = 45
	params['timetag_sec'] = 58
	params['stationid'] = 'A234567'
	params['stationloc_longitude'] = Decimal('-122.16328055555556')
	params['stationloc_latitude'] = Decimal('37.424458333333334')
	params['waterlevel'] = -97
	params['datum'] = 0
	params['sigma'] = -1.234
	params['o'] = 240
	params['levelinferred'] = False
	params['flat_tolerance_exceeded'] = True
	params['rate_tolerance_exceeded'] = False
	params['temp_tolerance_exceeded'] = False
	params['expected_height_exceeded'] = True
	params['link_down'] = False
	params['timeLastMeasured_month'] = 2
	params['timeLastMeasured_day'] = 28
	params['timeLastMeasured_hour'] = 23
	params['timeLastMeasured_min'] = 45
	params['timeLastMeasured_sec'] = 58

	return params

class Testwaterlevel(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the waterlevel message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['dac'],params['dac'])
		self.failUnlessEqual(r['fid'],params['fid'])
		self.failUnlessEqual(r['efid'],params['efid'])
		self.failUnlessEqual(r['timetag_month'],params['timetag_month'])
		self.failUnlessEqual(r['timetag_day'],params['timetag_day'])
		self.failUnlessEqual(r['timetag_hour'],params['timetag_hour'])
		self.failUnlessEqual(r['timetag_min'],params['timetag_min'])
		self.failUnlessEqual(r['timetag_sec'],params['timetag_sec'])
		self.failUnlessEqual(r['stationid'],params['stationid'])
		self.failUnlessAlmostEqual(r['stationloc_longitude'],params['stationloc_longitude'],5)
		self.failUnlessAlmostEqual(r['stationloc_latitude'],params['stationloc_latitude'],5)
		self.failUnlessEqual(r['waterlevel'],params['waterlevel'])
		self.failUnlessEqual(r['datum'],params['datum'])
		self.failUnlessAlmostEqual(r['sigma'],params['sigma'],3)
		self.failUnlessEqual(r['o'],params['o'])
		self.failUnlessEqual(r['levelinferred'],params['levelinferred'])
		self.failUnlessEqual(r['flat_tolerance_exceeded'],params['flat_tolerance_exceeded'])
		self.failUnlessEqual(r['rate_tolerance_exceeded'],params['rate_tolerance_exceeded'])
		self.failUnlessEqual(r['temp_tolerance_exceeded'],params['temp_tolerance_exceeded'])
		self.failUnlessEqual(r['expected_height_exceeded'],params['expected_height_exceeded'])
		self.failUnlessEqual(r['link_down'],params['link_down'])
		self.failUnlessEqual(r['timeLastMeasured_month'],params['timeLastMeasured_month'])
		self.failUnlessEqual(r['timeLastMeasured_day'],params['timeLastMeasured_day'])
		self.failUnlessEqual(r['timeLastMeasured_hour'],params['timeLastMeasured_hour'])
		self.failUnlessEqual(r['timeLastMeasured_min'],params['timeLastMeasured_min'])
		self.failUnlessEqual(r['timeLastMeasured_sec'],params['timeLastMeasured_sec'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "waterlevel" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "waterlevel" AIS message')
	parser.add_option('--timetag_month-field', dest='timetag_monthField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_day-field', dest='timetag_dayField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_hour-field', dest='timetag_hourField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_min-field', dest='timetag_minField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_sec-field', dest='timetag_secField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--stationid-field', dest='stationidField',default='@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--stationloc_longitude-field', dest='stationloc_longitudeField',default=Decimal('181'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--stationloc_latitude-field', dest='stationloc_latitudeField',default=Decimal('91'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--waterlevel-field', dest='waterlevelField',default=-32768,metavar='int',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--datum-field', dest='datumField',default=31,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--sigma-field', dest='sigmaField',metavar='float',type='float'
		,help='Field parameter value [default: %default]')
	parser.add_option('--o-field', dest='oField',default=255,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--levelinferred-field', dest='levelinferredField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--flat_tolerance_exceeded-field', dest='flat_tolerance_exceededField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--rate_tolerance_exceeded-field', dest='rate_tolerance_exceededField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--temp_tolerance_exceeded-field', dest='temp_tolerance_exceededField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--expected_height_exceeded-field', dest='expected_height_exceededField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--link_down-field', dest='link_downField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timeLastMeasured_month-field', dest='timeLastMeasured_monthField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timeLastMeasured_day-field', dest='timeLastMeasured_dayField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timeLastMeasured_hour-field', dest='timeLastMeasured_hourField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timeLastMeasured_min-field', dest='timeLastMeasured_minField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timeLastMeasured_sec-field', dest='timeLastMeasured_secField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv','sql' , 'kml','kml-full')
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	parser.add_option('-c','--sql-create',dest='sqlCreate',default=False,action='store_true',
			  help='Print out an sql create command for the table.')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.timetag_monthField: parser.error("missing value for timetag_monthField")
		if None==options.timetag_dayField: parser.error("missing value for timetag_dayField")
		if None==options.timetag_hourField: parser.error("missing value for timetag_hourField")
		if None==options.timetag_minField: parser.error("missing value for timetag_minField")
		if None==options.timetag_secField: parser.error("missing value for timetag_secField")
		if None==options.stationidField: parser.error("missing value for stationidField")
		if None==options.stationloc_longitudeField: parser.error("missing value for stationloc_longitudeField")
		if None==options.stationloc_latitudeField: parser.error("missing value for stationloc_latitudeField")
		if None==options.waterlevelField: parser.error("missing value for waterlevelField")
		if None==options.datumField: parser.error("missing value for datumField")
		if None==options.sigmaField: parser.error("missing value for sigmaField")
		if None==options.oField: parser.error("missing value for oField")
		if None==options.levelinferredField: parser.error("missing value for levelinferredField")
		if None==options.flat_tolerance_exceededField: parser.error("missing value for flat_tolerance_exceededField")
		if None==options.rate_tolerance_exceededField: parser.error("missing value for rate_tolerance_exceededField")
		if None==options.temp_tolerance_exceededField: parser.error("missing value for temp_tolerance_exceededField")
		if None==options.expected_height_exceededField: parser.error("missing value for expected_height_exceededField")
		if None==options.link_downField: parser.error("missing value for link_downField")
		if None==options.timeLastMeasured_monthField: parser.error("missing value for timeLastMeasured_monthField")
		if None==options.timeLastMeasured_dayField: parser.error("missing value for timeLastMeasured_dayField")
		if None==options.timeLastMeasured_hourField: parser.error("missing value for timeLastMeasured_hourField")
		if None==options.timeLastMeasured_minField: parser.error("missing value for timeLastMeasured_minField")
		if None==options.timeLastMeasured_secField: parser.error("missing value for timeLastMeasured_secField")
		msgDict={
			'dac': '366',
			'fid': '1',
			'efid': '1',
			'timetag_month': options.timetag_monthField,
			'timetag_day': options.timetag_dayField,
			'timetag_hour': options.timetag_hourField,
			'timetag_min': options.timetag_minField,
			'timetag_sec': options.timetag_secField,
			'stationid': options.stationidField,
			'stationloc_longitude': options.stationloc_longitudeField,
			'stationloc_latitude': options.stationloc_latitudeField,
			'waterlevel': options.waterlevelField,
			'datum': options.datumField,
			'sigma': options.sigmaField,
			'o': options.oField,
			'levelinferred': options.levelinferredField,
			'flat_tolerance_exceeded': options.flat_tolerance_exceededField,
			'rate_tolerance_exceeded': options.rate_tolerance_exceededField,
			'temp_tolerance_exceeded': options.temp_tolerance_exceededField,
			'expected_height_exceeded': options.expected_height_exceededField,
			'link_down': options.link_downField,
			'timeLastMeasured_month': options.timeLastMeasured_monthField,
			'timeLastMeasured_day': options.timeLastMeasured_dayField,
			'timeLastMeasured_hour': options.timeLastMeasured_hourField,
			'timeLastMeasured_min': options.timeLastMeasured_minField,
			'timeLastMeasured_sec': options.timeLastMeasured_secField,
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')


	if options.sqlCreate:
		sqlCreateStr(outfile,options.fieldList)

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType,fieldList=options.fieldList)
