#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-02-08 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = [
	'dac',
	'fid',
	'efid',
	'timetag_month',
	'timetag_day',
	'timetag_hour',
	'timetag_min',
	'timetag_sec',
	'stationid',
	'station_longitude',
	'station_latitude',
	'whale_longitude',
	'whale_latitude',
	'timetoexpire',
	'radius',
	'heading',
	'numWhales',
]

def encode(params, validate=False):
	'''Create a whalenotice binary message payload to pack into an AIS Msg whalenotice.

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (dac+fid+id defines which message) (field automatically set to "2")
	  - timetag_month(uint): Time whale was last recorded  month 1..12
	  - timetag_day(uint): Time whale was last recorded  day of the month 1..31
	  - timetag_hour(uint): Time whale was last recorded  UTC hours 0..23
	  - timetag_min(uint): Time whale was last recorded  minutes
	  - timetag_sec(uint): Time whale was last recorded  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - station_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - station_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - whale_longitude(decimal): Where was the whale  East West location
	  - whale_latitude(decimal): Where was the whale  North South location
	  - timetoexpire(uint): Seconds until this notice expires
	  - radius(uint): Distance of the required exclusion/slow zone
	  - heading(uint): Direction the whale is traveling
	  - numWhales(uint): Number of whales detected
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=366),16))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=2),12))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_month']),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_day']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_hour']),5))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_min']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetag_sec']),6))
	if 'stationid' in params:
		bvList.append(aisstring.encode(params['stationid'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'station_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['station_longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'station_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['station_latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'whale_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['whale_longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'whale_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['whale_latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'timetoexpire' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['timetoexpire']),16))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),16))
	if 'radius' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['radius']),16))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=65534),16))
	if 'heading' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['heading']),9))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=511),9))
	if 'numWhales' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['numWhales']),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),8))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a whalenotice message 

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (dac+fid+id defines which message) (field automatically set to "2")
	  - timetag_month(uint): Time whale was last recorded  month 1..12
	  - timetag_day(uint): Time whale was last recorded  day of the month 1..31
	  - timetag_hour(uint): Time whale was last recorded  UTC hours 0..23
	  - timetag_min(uint): Time whale was last recorded  minutes
	  - timetag_sec(uint): Time whale was last recorded  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - station_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - station_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - whale_longitude(decimal): Where was the whale  East West location
	  - whale_latitude(decimal): Where was the whale  North South location
	  - timetoexpire(uint): Seconds until this notice expires
	  - radius(uint): Distance of the required exclusion/slow zone
	  - heading(uint): Direction the whale is traveling
	  - numWhales(uint): Number of whales detected
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['dac']=366
	r['fid']=1
	r['efid']=2
	r['timetag_month']=int(bv[32:36])
	r['timetag_day']=int(bv[36:41])
	r['timetag_hour']=int(bv[41:46])
	r['timetag_min']=int(bv[46:52])
	r['timetag_sec']=int(bv[52:58])
	r['stationid']=aisstring.decode(bv[58:100])
	r['station_longitude']=Decimal(binary.signedIntFromBV(bv[100:128]))/Decimal('600000')
	r['station_latitude']=Decimal(binary.signedIntFromBV(bv[128:155]))/Decimal('600000')
	r['whale_longitude']=Decimal(binary.signedIntFromBV(bv[155:183]))/Decimal('600000')
	r['whale_latitude']=Decimal(binary.signedIntFromBV(bv[183:210]))/Decimal('600000')
	r['timetoexpire']=int(bv[210:226])
	r['radius']=int(bv[226:242])
	r['heading']=int(bv[242:251])
	r['numWhales']=int(bv[251:259])
	return r

def decodedac(bv, validate=False):
	return 366

def decodefid(bv, validate=False):
	return 1

def decodeefid(bv, validate=False):
	return 2

def decodetimetag_month(bv, validate=False):
	return int(bv[32:36])

def decodetimetag_day(bv, validate=False):
	return int(bv[36:41])

def decodetimetag_hour(bv, validate=False):
	return int(bv[41:46])

def decodetimetag_min(bv, validate=False):
	return int(bv[46:52])

def decodetimetag_sec(bv, validate=False):
	return int(bv[52:58])

def decodestationid(bv, validate=False):
	return aisstring.decode(bv[58:100])

def decodestation_longitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[100:128]))/Decimal('600000')

def decodestation_latitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[128:155]))/Decimal('600000')

def decodewhale_longitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[155:183]))/Decimal('600000')

def decodewhale_latitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[183:210]))/Decimal('600000')

def decodetimetoexpire(bv, validate=False):
	return int(bv[210:226])

def decoderadius(bv, validate=False):
	return int(bv[226:242])

def decodeheading(bv, validate=False):
	return int(bv[242:251])

def decodenumWhales(bv, validate=False):
	return int(bv[251:259])


def printHtml(params, out=sys.stdout):
		out.write("<h3>whalenotice<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dac</td>\n")
		out.write("<td>uint</td>\n")
		if 'dac' in params:
			out.write("	<td>"+str(params['dac'])+"</td>\n")
			out.write("	<td>"+str(params['dac'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>fid</td>\n")
		out.write("<td>uint</td>\n")
		if 'fid' in params:
			out.write("	<td>"+str(params['fid'])+"</td>\n")
			out.write("	<td>"+str(params['fid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>efid</td>\n")
		out.write("<td>uint</td>\n")
		if 'efid' in params:
			out.write("	<td>"+str(params['efid'])+"</td>\n")
			out.write("	<td>"+str(params['efid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_month</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_month' in params:
			out.write("	<td>"+str(params['timetag_month'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_month'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_day</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_day' in params:
			out.write("	<td>"+str(params['timetag_day'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_day'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_hour</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_hour' in params:
			out.write("	<td>"+str(params['timetag_hour'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_hour'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_min</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_min' in params:
			out.write("	<td>"+str(params['timetag_min'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_min'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetag_sec</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetag_sec' in params:
			out.write("	<td>"+str(params['timetag_sec'])+"</td>\n")
			out.write("	<td>"+str(params['timetag_sec'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>stationid</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'stationid' in params:
			out.write("	<td>"+str(params['stationid'])+"</td>\n")
			out.write("	<td>"+str(params['stationid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>station_longitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'station_longitude' in params:
			out.write("	<td>"+str(params['station_longitude'])+"</td>\n")
			out.write("	<td>"+str(params['station_longitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>station_latitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'station_latitude' in params:
			out.write("	<td>"+str(params['station_latitude'])+"</td>\n")
			out.write("	<td>"+str(params['station_latitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>whale_longitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'whale_longitude' in params:
			out.write("	<td>"+str(params['whale_longitude'])+"</td>\n")
			out.write("	<td>"+str(params['whale_longitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>whale_latitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'whale_latitude' in params:
			out.write("	<td>"+str(params['whale_latitude'])+"</td>\n")
			out.write("	<td>"+str(params['whale_latitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>timetoexpire</td>\n")
		out.write("<td>uint</td>\n")
		if 'timetoexpire' in params:
			out.write("	<td>"+str(params['timetoexpire'])+"</td>\n")
			out.write("	<td>"+str(params['timetoexpire'])+"</td>\n")
		out.write("<td>seconds</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>radius</td>\n")
		out.write("<td>uint</td>\n")
		if 'radius' in params:
			out.write("	<td>"+str(params['radius'])+"</td>\n")
			out.write("	<td>"+str(params['radius'])+"</td>\n")
		out.write("<td>m</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>heading</td>\n")
		out.write("<td>uint</td>\n")
		if 'heading' in params:
			out.write("	<td>"+str(params['heading'])+"</td>\n")
			out.write("	<td>"+str(params['heading'])+"</td>\n")
		out.write("<td>degrees true north</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>numWhales</td>\n")
		out.write("<td>uint</td>\n")
		if 'numWhales' in params:
			out.write("	<td>"+str(params['numWhales'])+"</td>\n")
			out.write("	<td>"+str(params['numWhales'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")


def printKml(params, out=sys.stdout):
	'''KML (Keyhole Markup Language) for Google Earth, but without the header/footer'''
	out.write("\	<Placemark>\n")
	out.write("\t	<name>"+str(params['stationsid'])+"</name>\n")
	out.write("\t\t<description>\n")
	import StringIO
	buf = StringIO.StringIO()
	printHtml(params,buf)
	import cgi
	out.write(cgi.escape(buf.getvalue()))
	out.write("\t\t</description>\n")
	out.write("\t\t<styleUrl>#m_ylw-pushpin_copy0</styleUrl>\n")
	out.write("\t\t<Point>\n")
	out.write("\t\t\t<coordinates>")
	out.write(str(params['station_longitude']))
	out.write(',')
	out.write(str(params['station_latitude']))
	out.write(",0</coordinates>\n")
	out.write("\t\t</Point>\n")
	out.write("\t</Placemark>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None):
	'''Print a numWhales message to stdout.

	Fields in params:
	  - dac(uint): Designated Area Code (field automatically set to "366")
	  - fid(uint): Functional Identifier (field automatically set to "1")
	  - efid(uint): extended functional identifier (dac+fid+id defines which message) (field automatically set to "2")
	  - timetag_month(uint): Time whale was last recorded  month 1..12
	  - timetag_day(uint): Time whale was last recorded  day of the month 1..31
	  - timetag_hour(uint): Time whale was last recorded  UTC hours 0..23
	  - timetag_min(uint): Time whale was last recorded  minutes
	  - timetag_sec(uint): Time whale was last recorded  seconds
	  - stationid(aisstr6): Character identifier of the station.  Usually a number.
	  - station_longitude(decimal): Location of the sensor taking the water level measurement or position of prediction  East West location
	  - station_latitude(decimal): Location of the sensor taking the water level measurement or position of prediction  North South location
	  - whale_longitude(decimal): Where was the whale  East West location
	  - whale_latitude(decimal): Where was the whale  North South location
	  - timetoexpire(uint): Seconds until this notice expires
	  - radius(uint): Distance of the required exclusion/slow zone
	  - heading(uint): Direction the whale is traveling
	  - numWhales(uint): Number of whales detected
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("numWhales:\n")
		if 'dac' in params: out.write("	dac:                "+str(params['dac'])+"\n")
		if 'fid' in params: out.write("	fid:                "+str(params['fid'])+"\n")
		if 'efid' in params: out.write("	efid:               "+str(params['efid'])+"\n")
		if 'timetag_month' in params: out.write("	timetag_month:      "+str(params['timetag_month'])+"\n")
		if 'timetag_day' in params: out.write("	timetag_day:        "+str(params['timetag_day'])+"\n")
		if 'timetag_hour' in params: out.write("	timetag_hour:       "+str(params['timetag_hour'])+"\n")
		if 'timetag_min' in params: out.write("	timetag_min:        "+str(params['timetag_min'])+"\n")
		if 'timetag_sec' in params: out.write("	timetag_sec:        "+str(params['timetag_sec'])+"\n")
		if 'stationid' in params: out.write("	stationid:          "+str(params['stationid'])+"\n")
		if 'station_longitude' in params: out.write("	station_longitude:  "+str(params['station_longitude'])+"\n")
		if 'station_latitude' in params: out.write("	station_latitude:   "+str(params['station_latitude'])+"\n")
		if 'whale_longitude' in params: out.write("	whale_longitude:    "+str(params['whale_longitude'])+"\n")
		if 'whale_latitude' in params: out.write("	whale_latitude:     "+str(params['whale_latitude'])+"\n")
		if 'timetoexpire' in params: out.write("	timetoexpire:       "+str(params['timetoexpire'])+"\n")
		if 'radius' in params: out.write("	radius:             "+str(params['radius'])+"\n")
		if 'heading' in params: out.write("	heading:            "+str(params['heading'])+"\n")
		if 'numWhales' in params: out.write("	numWhales:          "+str(params['numWhales'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	elif 'sql'==format:
		sqlInsertStr(params,out)
	elif 'kml'==format:
		printKml(params,out)
	elif 'kml-full'==format:
		out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		out.write("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n")
		out.write("<Document>\n")
		out.write("	<name>whalenotice</name>\n")
		printKml(params,out)
		out.write("</Document>\n")
		out.write("</kml>\n")
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

######################################################################
# SQL SUPPORT
######################################################################

def sqlCreateStr(outfile=sys.stdout, fields=None, extraFields=None, addCoastGuardFields=True):
	'''
	Return the SQL CREATE command for this message type
	@param outfile: file like object to print to.
	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlCreate(fields,extraFields,addCoastGuardFields)))

def sqlCreate(fields=None, extraFields=None, addCoastGuardFields=True):
	'''
	Return the sqlhelp object to create the table.

	@param fields: which fields to put in the create.  Defaults to all.
	@param extraFields: A sequence of tuples containing (name,sql type) for additional fields
	@param addCoastGuardFields: Add the extra fields that come after the NMEA check some from the USCG N-AIS format
	@type addCoastGuardFields: bool
	@return: An object that can be used to generate a return
	@rtype: sqlhelp.create
	'''
	if None == fields: fields = fieldList
	import sqlhelp
	c = sqlhelp.create('whalenotice')
	if 'dac' in fields: c.addInt ('dac')
	if 'fid' in fields: c.addInt ('fid')
	if 'efid' in fields: c.addInt ('efid')
	if 'timetag_month' in fields: c.addInt ('timetag_month')
	if 'timetag_day' in fields: c.addInt ('timetag_day')
	if 'timetag_hour' in fields: c.addInt ('timetag_hour')
	if 'timetag_min' in fields: c.addInt ('timetag_min')
	if 'timetag_sec' in fields: c.addInt ('timetag_sec')
	if 'stationid' in fields: c.addVarChar('stationid',7)
	if 'station_longitude' in fields: c.addDecimal('station_longitude',8,5)
	if 'station_latitude' in fields: c.addDecimal('station_latitude',8,5)
	if 'whale_longitude' in fields: c.addDecimal('whale_longitude',8,5)
	if 'whale_latitude' in fields: c.addDecimal('whale_latitude',8,5)
	if 'timetoexpire' in fields: c.addInt ('timetoexpire')
	if 'radius' in fields: c.addInt ('radius')
	if 'heading' in fields: c.addInt ('heading')
	if 'numWhales' in fields: c.addInt ('numWhales')

	if addCoastGuardFields:
		# c.addInt('cg_rssi')     # Relative signal strength indicator
		# c.addInt('cg_d')        # dBm receive strength
		# c.addInt('cg_T')        # Receive timestamp from the AIS equipment
		# c.addInt('cg_S')        # Slot received in
		# c.addVarChar('cg_x',10) # Idonno
		c.addVarChar('cg_r',15)   # Receiver station ID  -  should usually be an MMSI, but sometimes is a string
		c.addInt('cg_timestamp')  # UTC seconds since the epoch
		# FIX: maybe an actually time field?

	return c

def sqlInsertStr(params, outfile=sys.stdout, extraParams=None):
	'''
	Return the SQL CREATE command for this message type
	@param params: dictionary of values keyed by field name
	@param outfile: file like object to print to.
	@param extraParams: A sequence of tuples containing (name,sql type) for additional fields
	@return: sql create string
	@rtype: str

	@see: sqlCreate
	'''
	outfile.write(str(sqlInsert(params,extraParams)))


def sqlInsert(params,extraParams=None):
	'''
	Give the SQL insert statement
	@param params: dict keyed by field name of values
	@param extraParams: any extra fields that you have created beyond the normal ais message fields
	@rtype: sqlhelp.insert
	@return: insert class instance
	@todo: allow optional type checking of params?
	@warning: this will take invalid keys happily and do what???
	'''
	import sqlhelp
	i = sqlhelp.insert('whalenotice')
	for key in params: 
		#i.add(key,params[key])
		if type(params[key])==Decimal: i.add(key,float(params[key]))
		else: i.add(key,params[key])
	if None != extraParams:
		for key in extraParams: 
			i.add(key,extraParams[key])

	return i


######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['dac'] = 366
	params['fid'] = 1
	params['efid'] = 2
	params['timetag_month'] = 2
	params['timetag_day'] = 28
	params['timetag_hour'] = 23
	params['timetag_min'] = 45
	params['timetag_sec'] = 58
	params['stationid'] = 'A234567'
	params['station_longitude'] = Decimal('-122.16328055555556')
	params['station_latitude'] = Decimal('37.424458333333334')
	params['whale_longitude'] = Decimal('-122.16328055555556')
	params['whale_latitude'] = Decimal('37.424458333333334')
	params['timetoexpire'] = 1
	params['radius'] = 5000
	params['heading'] = 35
	params['numWhales'] = 2

	return params

class Testwhalenotice(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the whalenotice message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['dac'],params['dac'])
		self.failUnlessEqual(r['fid'],params['fid'])
		self.failUnlessEqual(r['efid'],params['efid'])
		self.failUnlessEqual(r['timetag_month'],params['timetag_month'])
		self.failUnlessEqual(r['timetag_day'],params['timetag_day'])
		self.failUnlessEqual(r['timetag_hour'],params['timetag_hour'])
		self.failUnlessEqual(r['timetag_min'],params['timetag_min'])
		self.failUnlessEqual(r['timetag_sec'],params['timetag_sec'])
		self.failUnlessEqual(r['stationid'],params['stationid'])
		self.failUnlessAlmostEqual(r['station_longitude'],params['station_longitude'],5)
		self.failUnlessAlmostEqual(r['station_latitude'],params['station_latitude'],5)
		self.failUnlessAlmostEqual(r['whale_longitude'],params['whale_longitude'],5)
		self.failUnlessAlmostEqual(r['whale_latitude'],params['whale_latitude'],5)
		self.failUnlessEqual(r['timetoexpire'],params['timetoexpire'])
		self.failUnlessEqual(r['radius'],params['radius'])
		self.failUnlessEqual(r['heading'],params['heading'])
		self.failUnlessEqual(r['numWhales'],params['numWhales'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "whalenotice" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "whalenotice" AIS message')
	parser.add_option('--timetag_month-field', dest='timetag_monthField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_day-field', dest='timetag_dayField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_hour-field', dest='timetag_hourField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_min-field', dest='timetag_minField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetag_sec-field', dest='timetag_secField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--stationid-field', dest='stationidField',default='@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--station_longitude-field', dest='station_longitudeField',default=Decimal('181'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--station_latitude-field', dest='station_latitudeField',default=Decimal('91'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--whale_longitude-field', dest='whale_longitudeField',default=Decimal('181'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--whale_latitude-field', dest='whale_latitudeField',default=Decimal('91'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--timetoexpire-field', dest='timetoexpireField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--radius-field', dest='radiusField',default=65534,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--heading-field', dest='headingField',default=511,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--numWhales-field', dest='numWhalesField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv','sql' , 'kml','kml-full')
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	parser.add_option('-c','--sql-create',dest='sqlCreate',default=False,action='store_true',
			  help='Print out an sql create command for the table.')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.timetag_monthField: parser.error("missing value for timetag_monthField")
		if None==options.timetag_dayField: parser.error("missing value for timetag_dayField")
		if None==options.timetag_hourField: parser.error("missing value for timetag_hourField")
		if None==options.timetag_minField: parser.error("missing value for timetag_minField")
		if None==options.timetag_secField: parser.error("missing value for timetag_secField")
		if None==options.stationidField: parser.error("missing value for stationidField")
		if None==options.station_longitudeField: parser.error("missing value for station_longitudeField")
		if None==options.station_latitudeField: parser.error("missing value for station_latitudeField")
		if None==options.whale_longitudeField: parser.error("missing value for whale_longitudeField")
		if None==options.whale_latitudeField: parser.error("missing value for whale_latitudeField")
		if None==options.timetoexpireField: parser.error("missing value for timetoexpireField")
		if None==options.radiusField: parser.error("missing value for radiusField")
		if None==options.headingField: parser.error("missing value for headingField")
		if None==options.numWhalesField: parser.error("missing value for numWhalesField")
		msgDict={
			'dac': '366',
			'fid': '1',
			'efid': '2',
			'timetag_month': options.timetag_monthField,
			'timetag_day': options.timetag_dayField,
			'timetag_hour': options.timetag_hourField,
			'timetag_min': options.timetag_minField,
			'timetag_sec': options.timetag_secField,
			'stationid': options.stationidField,
			'station_longitude': options.station_longitudeField,
			'station_latitude': options.station_latitudeField,
			'whale_longitude': options.whale_longitudeField,
			'whale_latitude': options.whale_latitudeField,
			'timetoexpire': options.timetoexpireField,
			'radius': options.radiusField,
			'heading': options.headingField,
			'numWhales': options.numWhalesField,
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')


	if options.sqlCreate:
		sqlCreateStr(outfile,options.fieldList)

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType,fieldList=options.fieldList)
